@@ -66,7 +66,7 @@ check_dir_listing(GConfEngine* conf)
   const gchar** iter;
   GSList* entries;
   GSList* iter2;
-  GConfError* error = NULL;
+  GError* error = NULL;
   gboolean found[sizeof(keys_in_foo)];
   int i;
   gboolean got_it;
@@ -80,13 +80,13 @@ check_dir_listing(GConfEngine* conf)
 
       tmp = g_strconcat(*iter, "/woo", NULL);
       
-      full_key = gconf_concat_key_and_dir("/testing/foo", tmp);
+      full_key = gconf_concat_dir_and_key("/testing/foo", tmp);
       g_free(tmp);
       
-      gconf_set_int(conf, full_key, 10, &error);
+      gconf_engine_set_int(conf, full_key, 10, &error);
 
       check (error == NULL, "Error setting key %s: %s",
-             full_key, error ? error->str : "");
+             full_key, error ? error->message : "");
       
       g_free(full_key);
 
@@ -94,17 +94,17 @@ check_dir_listing(GConfEngine* conf)
     }
 
   /* Check that "testing" comes back in a list of / */
-  entries = gconf_all_dirs(conf, "/", &error);
+  entries = gconf_engine_all_dirs(conf, "/", &error);
 
   check (error == NULL, "Error getting list of dirs in /: %s",
-         error ? error->str : "");
+         error ? error->message : "");
 
   got_it = FALSE;
   
   iter2 = entries;
   while (iter2 != NULL)
     {
-      if (strcmp(iter2->data, "testing") == 0)
+      if (strcmp(iter2->data, "/testing") == 0)
         got_it = TRUE;
       
       iter2 = g_slist_next(iter2);
@@ -114,17 +114,17 @@ check_dir_listing(GConfEngine* conf)
   check(got_it, "Did not get 'testing' in listing of / after setting keys such as /testing/foo/bar/woo");
   
   /* Check that "foo" comes back in a list of /testing */
-  entries = gconf_all_dirs(conf, "/testing", &error);
+  entries = gconf_engine_all_dirs(conf, "/testing", &error);
 
   check (error == NULL, "Error getting list of dirs in /testing: %s",
-         error ? error->str : "");
+         error ? error->message : "");
 
   got_it = FALSE;
   
   iter2 = entries;
   while (iter2 != NULL)
     {
-      if (strcmp(iter2->data, "foo") == 0)
+      if (strcmp(iter2->data, "/testing/foo") == 0)
         got_it = TRUE;
       
       iter2 = g_slist_next(iter2);
@@ -135,10 +135,10 @@ check_dir_listing(GConfEngine* conf)
   
   /* Check that /testing/foo/ subdirs come back in a listing of /testing/foo */
   
-  entries = gconf_all_dirs(conf, "/testing/foo", &error);
+  entries = gconf_engine_all_dirs(conf, "/testing/foo", &error);
 
   check (error == NULL, "Error getting list of dirs in /testing/foo: %s",
-         error ? error->str : "");
+         error ? error->message : "");
 
   iter2 = entries;
   while (iter2 != NULL)
@@ -147,9 +147,12 @@ check_dir_listing(GConfEngine* conf)
       iter = keys_in_foo;
       while (*iter)
         {
-          if (strcmp(iter2->data, *iter) == 0)
+          gchar *full = gconf_concat_dir_and_key ("/testing/foo", *iter);
+          if (strcmp(iter2->data, full) == 0)
             found[i] = TRUE;
-              
+
+          g_free (full);
+          
           ++i;
           ++iter;
         }
@@ -178,13 +181,13 @@ check_dir_listing(GConfEngine* conf)
 
       tmp = g_strconcat(*iter, "/woo", NULL);
 
-      full_key = gconf_concat_key_and_dir("/testing/foo", tmp);
+      full_key = gconf_concat_dir_and_key("/testing/foo", tmp);
       g_free(tmp);
       
-      gconf_unset(conf, full_key, &error);
+      gconf_engine_unset(conf, full_key, &error);
 
       check (error == NULL, "Error unsetting key %s: %s",
-             full_key, error ? error->str : "");
+             full_key, error ? error->message : "");
       
       g_free(full_key);
 
@@ -196,14 +199,14 @@ int
 main (int argc, char** argv)
 {
   GConfEngine* conf;
-  GConfError* err = NULL;
+  GError* err = NULL;
   
   if (!gconf_init(argc, argv, &err))
     {
       g_assert(err != NULL);
-      fprintf(stderr, "Failed to init GConf: %s\n", err->str);
+      fprintf(stderr, "Failed to init GConf: %s\n", err->message);
       fflush(stderr);
-      gconf_error_destroy(err);
+      g_error_free(err);
       err = NULL;
       return 1;
     }
